@@ -11,11 +11,15 @@ use casper_node::{
     rpcs::{account::PutDeploy, chain::GetBlockResult, info::GetDeploy, RpcWithParams},
     types::{Deploy, DeployHash, TimeDiff, Timestamp},
 };
-use casper_types::{ProtocolVersion, RuntimeArgs, SecretKey, URef, U512};
+use casper_types::{
+    account::AccountHash, AsymmetricType, ProtocolVersion, PublicKey, RuntimeArgs, SecretKey,
+    UIntParseError, URef, U512,
+};
 
 use crate::{
     error::{Error, Result},
-    rpc::{RpcClient, TransferTarget},
+    parsing,
+    rpc::RpcClient,
 };
 
 /// The maximum permissible size in bytes of a Deploy when serialized via `ToBytes`.
@@ -171,6 +175,9 @@ pub struct DeployParams {
 
     /// The name of the chain this `Deploy` will be considered for inclusion in.
     pub chain_name: String,
+
+    /// Optional public key of the account creating the Deploy.
+    pub session_account: Option<PublicKey>,
 }
 
 /// An extension trait that adds some client-specific functionality to `Deploy`.
@@ -184,10 +191,10 @@ pub(super) trait DeployExt {
 
     /// Constructs a transfer `Deploy`.
     fn new_transfer(
-        amount: U512,
+        amount: &str,
         source_purse: Option<URef>,
-        target: TransferTarget,
-        transfer_id: u64,
+        target_account: &str,
+        transfer_id: &str,
         params: DeployParams,
         payment: ExecutableDeployItem,
     ) -> Result<Deploy>;
@@ -222,6 +229,7 @@ impl DeployExt for Deploy {
             dependencies,
             chain_name,
             secret_key,
+            session_account,
         } = params;
 
         let deploy = Deploy::new(
@@ -233,16 +241,17 @@ impl DeployExt for Deploy {
             payment,
             session,
             &secret_key,
+            session_account,
         );
         deploy.is_valid_size(MAX_SERIALIZED_SIZE)?;
         Ok(deploy)
     }
 
     fn new_transfer(
-        amount: U512,
+        amount: &str,
         source_purse: Option<URef>,
-        target: TransferTarget,
-        transfer_id: u64,
+        target_account: &str,
+        transfer_id: &str,
         params: DeployParams,
         payment: ExecutableDeployItem,
     ) -> Result<Deploy> {
@@ -251,22 +260,42 @@ impl DeployExt for Deploy {
         const TRANSFER_ARG_TARGET: &str = "target";
         const TRANSFER_ARG_ID: &str = "id";
 
+        let amount = U512::from_dec_str(amount).map_err(|err| Error::FailedToParseUint {
+            context: TRANSFER_ARG_AMOUNT,
+            error: UIntParseError::FromDecStr(err),
+        })?;
+
         let mut transfer_args = RuntimeArgs::new();
         transfer_args.insert(TRANSFER_ARG_AMOUNT, amount)?;
+
         if let Some(source_purse) = source_purse {
             transfer_args.insert(TRANSFER_ARG_SOURCE, source_purse)?;
         }
-        match target {
-            TransferTarget::Account(target_account) => {
-                let target_account_hash = target_account.to_account_hash().value();
-                transfer_args.insert(TRANSFER_ARG_TARGET, target_account_hash)?;
-            }
+
+        if let Ok(public_key) = PublicKey::from_hex(target_account) {
+            transfer_args.insert(TRANSFER_ARG_TARGET, public_key)?;
+        } else if let Ok(account_hash) = AccountHash::from_formatted_str(target_account) {
+            transfer_args.insert(TRANSFER_ARG_TARGET, account_hash)?;
+        } else if let Ok(uref) = URef::from_formatted_str(target_account) {
+            transfer_args.insert(TRANSFER_ARG_TARGET, uref)?;
+        } else {
+            return Err(Error::InvalidArgument {
+                context: "target_account",
+                error: format!(
+                    "Allowed types: PublicKey, AccountHash or URef, got {}",
+                    target_account
+                ),
+            });
         }
+
+        let transfer_id = parsing::transfer_id(transfer_id)?;
         let maybe_transfer_id = Some(transfer_id);
         transfer_args.insert(TRANSFER_ARG_ID, maybe_transfer_id)?;
+
         let session = ExecutableDeployItem::Transfer {
             args: transfer_args,
         };
+
         Deploy::with_payment_and_session(params, payment, session)
     }
 
@@ -394,15 +423,6 @@ mod tests {
       ]
     }"#;
 
-    #[derive(Debug)]
-    struct ErrWrapper(pub Error);
-
-    impl PartialEq for ErrWrapper {
-        fn eq(&self, other: &ErrWrapper) -> bool {
-            format!("{:?}", self.0) == format!("{:?}", other.0)
-        }
-    }
-
     pub fn deploy_params() -> DeployStrParams<'static> {
         DeployStrParams {
             secret_key: "../resources/local/secret_keys/node-1.pem",
@@ -417,6 +437,12 @@ mod tests {
         }
     }
 
+    pub fn malformed_deploy_params() -> DeployStrParams<'static> {
+        let mut params = deploy_params();
+        params.session_account = "incorrect string";
+        params
+    }
+
     fn args_simple() -> Vec<&'static str> {
         vec!["name_01:bool='false'", "name_02:i32='42'"]
     }
@@ -494,10 +520,7 @@ mod tests {
     #[test]
     fn should_read_deploy() {
         let bytes = SAMPLE_DEPLOY.as_bytes();
-        assert_eq!(
-            Deploy::read_deploy(bytes).map(|_| ()).map_err(ErrWrapper),
-            Ok(())
-        );
+        assert!(matches!(Deploy::read_deploy(bytes), Ok(_)));
     }
 
     #[test]
@@ -525,4 +548,87 @@ mod tests {
             signed_deploy
         );
     }
+
+    #[test]
+    fn should_create_transfer() {
+        use casper_types::{AsymmetricType, PublicKey};
+
+        // with public key.
+        let secret_key = SecretKey::generate_ed25519().unwrap();
+        let public_key = PublicKey::from(&secret_key).to_hex();
+        let transfer_deploy = Deploy::new_transfer(
+            "10000",
+            None,
+            &public_key,
+            "1",
+            deploy_params().try_into().unwrap(),
+            ExecutableDeployItem::Transfer {
+                args: RuntimeArgs::default(),
+            },
+        );
+
+        assert!(transfer_deploy.is_ok());
+        assert!(transfer_deploy.unwrap().session().is_transfer());
+
+        // with account hash
+        let account_hash =
+            "account-hash-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20";
+        let transfer_deploy = Deploy::new_transfer(
+            "10000",
+            None,
+            account_hash,
+            "1",
+            deploy_params().try_into().unwrap(),
+            ExecutableDeployItem::Transfer {
+                args: RuntimeArgs::default(),
+            },
+        );
+
+        assert!(transfer_deploy.is_ok());
+        assert!(transfer_deploy.unwrap().session().is_transfer());
+
+        // with uref.
+        let uref = "uref-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20-007";
+        let transfer_deploy = Deploy::new_transfer(
+            "10000",
+            None,
+            uref,
+            "1",
+            deploy_params().try_into().unwrap(),
+            ExecutableDeployItem::Transfer {
+                args: RuntimeArgs::default(),
+            },
+        );
+
+        assert!(transfer_deploy.is_ok());
+        assert!(transfer_deploy.unwrap().session().is_transfer());
+    }
+
+    #[test]
+    fn should_fail_to_create_transfer_with_bad_args() {
+        let transfer_deploy = Deploy::new_transfer(
+            "10000",
+            None,
+            "bad public key.",
+            "1",
+            deploy_params().try_into().unwrap(),
+            ExecutableDeployItem::Transfer {
+                args: RuntimeArgs::default(),
+            },
+        );
+
+        assert!(matches!(
+            transfer_deploy,
+            Err(Error::InvalidArgument {
+                context: "target_account",
+                error: _
+            })
+        ));
+    }
+
+    #[test]
+    #[should_panic]
+    fn should_fail_to_create_deploy_params() {
+        TryInto::<DeployParams>::try_into(malformed_deploy_params()).unwrap();
+    }
 }
