@@ -21,11 +21,11 @@ fn set_last_error(error: Error) {
     *last_error = Some(error)
 }
 
-/// FFI representation of [super::Error](super::Error)
+/// FFI representation of [Error](super::Error)
 ///
-/// The full error can be extracted with get_last_error.
-/// See [super::Error](super::Error) for more details on what these mean.
-#[allow(non_snake_case, non_camel_case_types, missing_docs)]
+/// The full error can be extracted with get_last_error. See [Error](super::Error) for more details
+/// on what these mean.
+#[allow(non_camel_case_types, missing_docs)]
 #[repr(C)]
 pub enum casper_error_t {
     CASPER_SUCCESS = 0,
@@ -54,6 +54,7 @@ pub enum casper_error_t {
     CASPER_CONFLICTING_ARGUMENTS = -23,
     CASPER_DEPLOY_SIZE_TOO_LARGE = -24,
     CASPER_FAILED_TO_CREATE_DICTIONARY_IDENTIFIER = -25,
+    CASPER_FAILED_TO_PARSE_STATE_IDENTIFIER = -26,
 }
 
 trait AsFFIError {
@@ -64,11 +65,13 @@ impl AsFFIError for Error {
     fn as_ffi_error(&self) -> casper_error_t {
         match self {
             Error::FailedToParseKey => casper_error_t::CASPER_FAILED_TO_PARSE_KEY,
-            Error::FailedToParseURef(_, _) => casper_error_t::CASPER_FAILED_TO_PARSE_UREF,
-            Error::FailedToParseInt(_, _) => casper_error_t::CASPER_FAILED_TO_PARSE_INT,
-            Error::FailedToParseTimeDiff(_, _) => casper_error_t::CASPER_FAILED_TO_PARSE_TIME_DIFF,
-            Error::FailedToParseTimestamp(_, _) => casper_error_t::CASPER_FAILED_TO_PARSE_TIMESTAMP,
-            Error::FailedToParseUint(_, _) => casper_error_t::CASPER_FAILED_TO_PARSE_UINT,
+            Error::FailedToParseURef { .. } => casper_error_t::CASPER_FAILED_TO_PARSE_UREF,
+            Error::FailedToParseInt { .. } => casper_error_t::CASPER_FAILED_TO_PARSE_INT,
+            Error::FailedToParseTimeDiff { .. } => casper_error_t::CASPER_FAILED_TO_PARSE_TIME_DIFF,
+            Error::FailedToParseTimestamp { .. } => {
+                casper_error_t::CASPER_FAILED_TO_PARSE_TIMESTAMP
+            }
+            Error::FailedToParseUint { .. } => casper_error_t::CASPER_FAILED_TO_PARSE_UINT,
             Error::FailedToGetResponse(_) => casper_error_t::CASPER_FAILED_TO_GET_RESPONSE,
             Error::FailedToParseResponse(_) => casper_error_t::CASPER_FAILED_TO_PARSE_RESPONSE,
             Error::FileAlreadyExists(_) => casper_error_t::CASPER_FILE_ALREADY_EXISTS,
@@ -81,7 +84,7 @@ impl AsFFIError for Error {
             Error::ToBytesError(_) => casper_error_t::CASPER_TO_BYTES_ERROR,
             Error::CryptoError { .. } => casper_error_t::CASPER_CRYPTO_ERROR,
             Error::InvalidCLValue(_) => casper_error_t::CASPER_INVALID_CL_VALUE,
-            Error::InvalidArgument(_, _) => casper_error_t::CASPER_INVALID_ARGUMENT,
+            Error::InvalidArgument { .. } => casper_error_t::CASPER_INVALID_ARGUMENT,
             Error::InvalidResponse(_) => casper_error_t::CASPER_INVALID_RESPONSE,
             Error::FFISetupNotCalled => casper_error_t::CASPER_FFI_SETUP_NOT_CALLED,
             Error::FFIPtrNullButRequired(_) => casper_error_t::CASPER_FFI_PTR_NULL_BUT_REQUIRED,
@@ -90,6 +93,9 @@ impl AsFFIError for Error {
             Error::FailedToParseDictionaryIdentifier => {
                 casper_error_t::CASPER_FAILED_TO_CREATE_DICTIONARY_IDENTIFIER
             }
+            Error::FailedToParseStateIdentifier => {
+                casper_error_t::CASPER_FAILED_TO_PARSE_STATE_IDENTIFIER
+            }
         }
     }
 }
@@ -166,15 +172,13 @@ fn unsafe_str_arg(arg: *const c_char, arg_name: &'static str) -> Result<&'static
         }
         CStr::from_ptr(arg).to_str()
     }
-    .map_err(|error| {
-        Error::InvalidArgument(
-            arg_name,
-            format!(
-                "invalid utf8 value passed for arg '{}': {:?}",
-                stringify!($arg),
-                error,
-            ),
-        )
+    .map_err(|error| Error::InvalidArgument {
+        context: arg_name,
+        error: format!(
+            "invalid utf8 value passed for arg '{}': {:?}",
+            stringify!($arg),
+            error,
+        ),
     })
 }
 
@@ -256,7 +260,7 @@ pub extern "C" fn casper_get_last_error(buf: *mut c_uchar, len: usize) -> usize
 
 /// Creates a `Deploy` and sends it to the network for execution.
 ///
-/// See [super::put_deploy](super::put_deploy) for more details.
+/// See [put_deploy](super::put_deploy) for more details.
 #[no_mangle]
 pub extern "C" fn casper_put_deploy(
     maybe_rpc_id: *const c_char,
@@ -283,7 +287,8 @@ pub extern "C" fn casper_put_deploy(
             deploy_params,
             session_params,
             payment_params,
-        );
+        )
+        .await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -292,7 +297,7 @@ pub extern "C" fn casper_put_deploy(
 
 /// Creates a `Deploy` and outputs it to a file or stdout.
 ///
-/// See [super::make_deploy](super::make_deploy) for more details.
+/// See [make_deploy](super::make_deploy) for more details.
 #[no_mangle]
 pub extern "C" fn casper_make_deploy(
     maybe_output_path: *const c_char,
@@ -319,7 +324,7 @@ pub extern "C" fn casper_make_deploy(
 /// Reads a previously-saved `Deploy` from a file, cryptographically signs it, and outputs it to a
 /// file or stdout.
 ///
-/// See [super::sign_deploy_file](super::sign_deploy_file) for more details.
+/// See [sign_deploy_file](super::sign_deploy_file) for more details.
 #[no_mangle]
 pub extern "C" fn casper_sign_deploy_file(
     input_path: *const c_char,
@@ -337,7 +342,7 @@ pub extern "C" fn casper_sign_deploy_file(
 
 /// Reads a previously-saved `Deploy` from a file and sends it to the network for execution.
 ///
-/// See [super::send_deploy_file](super::send_deploy_file) for more details.
+/// See [send_deploy_file](super::send_deploy_file) for more details.
 #[no_mangle]
 pub extern "C" fn casper_send_deploy_file(
     maybe_rpc_id: *const c_char,
@@ -354,7 +359,7 @@ pub extern "C" fn casper_send_deploy_file(
     let input_path = try_unsafe_arg!(input_path);
     runtime.block_on(async move {
         let result =
-            super::send_deploy_file(maybe_rpc_id, node_address, verbosity_level, input_path);
+            super::send_deploy_file(maybe_rpc_id, node_address, verbosity_level, input_path).await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -363,14 +368,14 @@ pub extern "C" fn casper_send_deploy_file(
 
 /// Transfers funds between purses.
 ///
-/// See [super::transfer](super::transfer) for more details.
+/// See [transfer](super::transfer) for more details.
 #[no_mangle]
 pub extern "C" fn casper_transfer(
     maybe_rpc_id: *const c_char,
     node_address: *const c_char,
     verbosity_level: u64,
     amount: *const c_char,
-    maybe_target_account: *const c_char,
+    target_account: *const c_char,
     transfer_id: *const c_char,
     deploy_params: *const casper_deploy_params_t,
     payment_params: *const casper_payment_params_t,
@@ -382,7 +387,7 @@ pub extern "C" fn casper_transfer(
     let maybe_rpc_id = try_unsafe_arg!(maybe_rpc_id);
     let node_address = try_unsafe_arg!(node_address);
     let amount = try_unsafe_arg!(amount);
-    let maybe_target_account = try_unsafe_arg!(maybe_target_account);
+    let target_account = try_unsafe_arg!(target_account);
     let transfer_id = try_unsafe_arg!(transfer_id);
     let deploy_params = try_arg_into!(deploy_params);
     let payment_params = try_arg_into!(payment_params);
@@ -392,11 +397,12 @@ pub extern "C" fn casper_transfer(
             node_address,
             verbosity_level,
             amount,
-            maybe_target_account,
+            target_account,
             transfer_id,
             deploy_params,
             payment_params,
-        );
+        )
+        .await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -405,12 +411,12 @@ pub extern "C" fn casper_transfer(
 
 /// Creates a transfer `Deploy` and outputs it to a file or stdout.
 ///
-/// See [super::make_transfer](super::make_transfer) for more details.
+/// See [make_transfer](super::make_transfer) for more details.
 #[no_mangle]
 pub extern "C" fn casper_make_transfer(
     maybe_output_path: *const c_char,
     amount: *const c_char,
-    maybe_target_account: *const c_char,
+    target_account: *const c_char,
     transfer_id: *const c_char,
     deploy_params: *const casper_deploy_params_t,
     payment_params: *const casper_payment_params_t,
@@ -418,14 +424,14 @@ pub extern "C" fn casper_make_transfer(
 ) -> casper_error_t {
     let maybe_output_path = try_unsafe_arg!(maybe_output_path);
     let amount = try_unsafe_arg!(amount);
-    let maybe_target_account = try_unsafe_arg!(maybe_target_account);
+    let target_account = try_unsafe_arg!(target_account);
     let transfer_id = try_unsafe_arg!(transfer_id);
     let deploy_params = try_arg_into!(deploy_params);
     let payment_params = try_arg_into!(payment_params);
     let result = super::make_transfer(
         maybe_output_path,
         amount,
-        maybe_target_account,
+        target_account,
         transfer_id,
         deploy_params,
         payment_params,
@@ -437,7 +443,7 @@ pub extern "C" fn casper_make_transfer(
 
 /// Retrieves a `Deploy` from the network.
 ///
-/// See [super::get_deploy](super::get_deploy) for more details.
+/// See [get_deploy](super::get_deploy) for more details.
 #[no_mangle]
 pub extern "C" fn casper_get_deploy(
     maybe_rpc_id: *const c_char,
@@ -453,7 +459,8 @@ pub extern "C" fn casper_get_deploy(
     let node_address = try_unsafe_arg!(node_address);
     let deploy_hash = try_unsafe_arg!(deploy_hash);
     runtime.block_on(async move {
-        let result = super::get_deploy(maybe_rpc_id, node_address, verbosity_level, deploy_hash);
+        let result =
+            super::get_deploy(maybe_rpc_id, node_address, verbosity_level, deploy_hash).await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -462,7 +469,7 @@ pub extern "C" fn casper_get_deploy(
 
 /// Retrieves a `Block` from the network.
 ///
-/// See [super::get_block](super::get_block) for more details.
+/// See [get_block](super::get_block) for more details.
 #[no_mangle]
 pub extern "C" fn casper_get_block(
     maybe_rpc_id: *const c_char,
@@ -478,7 +485,8 @@ pub extern "C" fn casper_get_block(
     let node_address = try_unsafe_arg!(node_address);
     let maybe_block_id = try_unsafe_arg!(maybe_block_id);
     runtime.block_on(async move {
-        let result = super::get_block(maybe_rpc_id, node_address, verbosity_level, maybe_block_id);
+        let result =
+            super::get_block(maybe_rpc_id, node_address, verbosity_level, maybe_block_id).await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -487,7 +495,7 @@ pub extern "C" fn casper_get_block(
 
 /// Retrieves all `Transfer` items for a `Block` from the network.
 ///
-/// See [super::casper_get_block_transfers](super::casper_get_block_transfers) for more details.
+/// See [get_block_transfers](super::get_block_transfers) for more details.
 #[no_mangle]
 pub extern "C" fn casper_get_block_transfers(
     maybe_rpc_id: *const c_char,
@@ -504,7 +512,8 @@ pub extern "C" fn casper_get_block_transfers(
     let maybe_block_id = try_unsafe_arg!(maybe_block_id);
     runtime.block_on(async move {
         let result =
-            super::get_block_transfers(maybe_rpc_id, node_address, verbosity_level, maybe_block_id);
+            super::get_block_transfers(maybe_rpc_id, node_address, verbosity_level, maybe_block_id)
+                .await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -513,7 +522,7 @@ pub extern "C" fn casper_get_block_transfers(
 
 /// Retrieves a state root hash at a given `Block`.
 ///
-/// See [super::get_state_root_hash](super::get_state_root_hash) for more details.
+/// See [get_state_root_hash](super::get_state_root_hash) for more details.
 #[no_mangle]
 pub extern "C" fn casper_get_state_root_hash(
     maybe_rpc_id: *const c_char,
@@ -530,7 +539,8 @@ pub extern "C" fn casper_get_state_root_hash(
     let maybe_block_id = try_unsafe_arg!(maybe_block_id);
     runtime.block_on(async move {
         let result =
-            super::get_state_root_hash(maybe_rpc_id, node_address, verbosity_level, maybe_block_id);
+            super::get_state_root_hash(maybe_rpc_id, node_address, verbosity_level, maybe_block_id)
+                .await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -539,8 +549,9 @@ pub extern "C" fn casper_get_state_root_hash(
 
 /// Retrieves a stored value from the network.
 ///
-/// See [super::get_item](super::get_item) for more details.
+/// See [get_item](super::get_item) for more details.
 #[no_mangle]
+#[deprecated(note = "Users should use `casper_client::query_global_state` instead.")]
 pub extern "C" fn casper_get_item(
     maybe_rpc_id: *const c_char,
     node_address: *const c_char,
@@ -559,6 +570,7 @@ pub extern "C" fn casper_get_item(
     let key = try_unsafe_arg!(key);
     let path = try_unsafe_arg!(path);
     runtime.block_on(async move {
+        #[allow(deprecated)]
         let result = super::get_item(
             maybe_rpc_id,
             node_address,
@@ -566,7 +578,79 @@ pub extern "C" fn casper_get_item(
             state_root_hash,
             key,
             path,
-        );
+        )
+        .await;
+        let response = try_unwrap_rpc!(result);
+        copy_str_to_buf(&response, response_buf, response_buf_len);
+        casper_error_t::CASPER_SUCCESS
+    })
+}
+
+/// Retrieves a stored value from a dictionary.
+///
+/// See [get_dictionary_item](super::get_dictionary_item) for more details.
+#[no_mangle]
+pub extern "C" fn casper_get_dictionary_item(
+    maybe_rpc_id: *const c_char,
+    node_address: *const c_char,
+    verbosity_level: u64,
+    state_root_hash: *const c_char,
+    dictionary_str_params: *const casper_dictionary_params_t,
+    response_buf: *mut c_uchar,
+    response_buf_len: usize,
+) -> casper_error_t {
+    let mut runtime = RUNTIME.lock().expect("should lock");
+    let runtime = try_unwrap_option!(&mut *runtime, or_else => Error::FFISetupNotCalled);
+    let maybe_rpc_id = try_unsafe_arg!(maybe_rpc_id);
+    let node_address = try_unsafe_arg!(node_address);
+    let state_root_hash = try_unsafe_arg!(state_root_hash);
+    let dictionary_str_params = try_arg_into!(dictionary_str_params);
+    runtime.block_on(async move {
+        let result = super::get_dictionary_item(
+            maybe_rpc_id,
+            node_address,
+            verbosity_level,
+            state_root_hash,
+            dictionary_str_params,
+        )
+        .await;
+        let response = try_unwrap_rpc!(result);
+        copy_str_to_buf(&response, response_buf, response_buf_len);
+        casper_error_t::CASPER_SUCCESS
+    })
+}
+
+/// Retrieves information from global state using either a Block hash or a state root hash.
+///
+/// See [query_global_state](super::query_global_state) for more info.
+#[no_mangle]
+pub extern "C" fn casper_query_global_state(
+    maybe_rpc_id: *const c_char,
+    node_address: *const c_char,
+    verbosity_level: u64,
+    global_state_params: *const casper_global_state_params_t,
+    key: *const c_char,
+    path: *const c_char,
+    response_buf: *mut c_uchar,
+    response_buf_len: usize,
+) -> casper_error_t {
+    let mut runtime = RUNTIME.lock().expect("should lock");
+    let runtime = try_unwrap_option!(&mut *runtime, or_else => Error::FFISetupNotCalled);
+    let maybe_rpc_id = try_unsafe_arg!(maybe_rpc_id);
+    let node_address = try_unsafe_arg!(node_address);
+    let global_state_params = try_arg_into!(global_state_params);
+    let key = try_unsafe_arg!(key);
+    let path = try_unsafe_arg!(path);
+    runtime.block_on(async move {
+        let result = super::query_global_state(
+            maybe_rpc_id,
+            node_address,
+            verbosity_level,
+            global_state_params,
+            key,
+            path,
+        )
+        .await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -575,7 +659,7 @@ pub extern "C" fn casper_get_item(
 
 /// Retrieves a purse's balance from the network.
 ///
-/// See [super::get_balance](super::get_balance) for more details.
+/// See [get_balance](super::get_balance) for more details.
 #[no_mangle]
 pub extern "C" fn casper_get_balance(
     maybe_rpc_id: *const c_char,
@@ -599,7 +683,8 @@ pub extern "C" fn casper_get_balance(
             verbosity_level,
             state_root_hash,
             purse,
-        );
+        )
+        .await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -608,7 +693,7 @@ pub extern "C" fn casper_get_balance(
 
 /// Retrieves era information from the network.
 ///
-/// See [super::get_era_info_by_switch_block](super::get_era_info_by_switch_block) for more details.
+/// See [get_era_info_by_switch_block](super::get_era_info_by_switch_block) for more details.
 #[no_mangle]
 pub extern "C" fn casper_get_era_info_by_switch_block(
     maybe_rpc_id: *const c_char,
@@ -629,7 +714,8 @@ pub extern "C" fn casper_get_era_info_by_switch_block(
             node_address,
             verbosity_level,
             maybe_block_id,
-        );
+        )
+        .await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -638,7 +724,7 @@ pub extern "C" fn casper_get_era_info_by_switch_block(
 
 /// Retrieves the bids and validators as of the most recently added `Block`.
 ///
-/// See [super::get_auction_info](super::get_auction_info) for more details.
+/// See [get_auction_info](super::get_auction_info) for more details.
 #[no_mangle]
 pub extern "C" fn casper_get_auction_info(
     maybe_rpc_id: *const c_char,
@@ -655,17 +741,57 @@ pub extern "C" fn casper_get_auction_info(
     let maybe_block_id = try_unsafe_arg!(maybe_block_id);
     runtime.block_on(async move {
         let result =
-            super::get_auction_info(maybe_rpc_id, node_address, verbosity_level, maybe_block_id);
+            super::get_auction_info(maybe_rpc_id, node_address, verbosity_level, maybe_block_id)
+                .await;
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
     })
 }
 
+/// Retrieves status changes of active validators.
+///
+/// See [get_validator_changes](super::get_validator_changes) for more details.
+#[no_mangle]
+pub extern "C" fn casper_get_validator_changes(
+    maybe_rpc_id: *const c_char,
+    node_address: *const c_char,
+    verbosity_level: u64,
+    response_buf: *mut c_uchar,
+    response_buf_len: usize,
+) -> casper_error_t {
+    let mut runtime = RUNTIME.lock().expect("should lock");
+    let runtime = try_unwrap_option!(&mut *runtime, or_else => Error::FFISetupNotCalled);
+    let maybe_rpc_id = try_unsafe_arg!(maybe_rpc_id);
+    let node_address = try_unsafe_arg!(node_address);
+    runtime.block_on(async move {
+        let result =
+            super::get_validator_changes(maybe_rpc_id, node_address, verbosity_level).await;
+        let response = try_unwrap_rpc!(result);
+        copy_str_to_buf(&response, response_buf, response_buf_len);
+        casper_error_t::CASPER_SUCCESS
+    })
+}
+
+/// Generates key files.
+///
+/// See [keygen::generate_files](super::keygen::generate_files) for more details.
+#[no_mangle]
+pub extern "C" fn casper_keygen(
+    output_dir: *const c_char,
+    algorithm: *const c_char,
+    force: bool,
+) -> casper_error_t {
+    let output_dir = try_unsafe_arg!(output_dir);
+    let algorithm = try_unsafe_arg!(algorithm);
+    let result = super::keygen::generate_files(output_dir, algorithm, force);
+    try_unwrap_result!(result);
+    casper_error_t::CASPER_SUCCESS
+}
+
 /// Container for `Deploy` construction options.
 ///
 /// See [DeployStrParams](super::DeployStrParams) for more info.
-#[allow(non_snake_case)]
 #[repr(C)]
 #[derive(Clone)]
 pub struct casper_deploy_params_t {
@@ -676,6 +802,7 @@ pub struct casper_deploy_params_t {
     dependencies: *const *const c_char,
     dependencies_len: usize,
     chain_name: *const c_char,
+    session_account: *const c_char,
 }
 
 impl TryInto<super::DeployStrParams<'_>> for casper_deploy_params_t {
@@ -692,13 +819,18 @@ impl TryInto<super::DeployStrParams<'_>> for casper_deploy_params_t {
             self.dependencies_len,
             "casper_deploy_params_t.dependencies",
         )?;
+        let session_account = unsafe_str_arg(
+            self.session_account,
+            "casper_deploy_params_t.session_account",
+        )?;
         Ok(super::DeployStrParams {
             secret_key,
             timestamp,
             ttl,
             gas_price,
-            chain_name,
             dependencies,
+            chain_name,
+            session_account,
         })
     }
 }
@@ -706,7 +838,6 @@ impl TryInto<super::DeployStrParams<'_>> for casper_deploy_params_t {
 /// Container for `Payment` construction options.
 ///
 /// See [PaymentStrParams](super::PaymentStrParams) for more info.
-#[allow(non_snake_case)]
 #[repr(C)]
 #[derive(Clone)]
 pub struct casper_payment_params_t {
@@ -780,7 +911,6 @@ impl TryInto<super::PaymentStrParams<'static>> for casper_payment_params_t {
 /// Container for `Session` construction options.
 ///
 /// See [SessionStrParams](super::SessionStrParams) for more info.
-#[allow(non_snake_case)]
 #[repr(C)]
 #[derive(Clone)]
 pub struct casper_session_params_t {
@@ -846,3 +976,121 @@ impl TryInto<super::SessionStrParams<'static>> for casper_session_params_t {
         })
     }
 }
+
+/// The two ways to construct a query to global state.
+///
+/// See [GlobalStateStrParams](super::GlobalStateStrParams) for more info.
+#[repr(C)]
+#[derive(Clone)]
+pub struct casper_global_state_params_t {
+    is_block_hash: bool,
+    hash_value: *const c_char,
+}
+
+impl TryInto<super::GlobalStateStrParams<'static>> for casper_global_state_params_t {
+    type Error = Error;
+
+    fn try_into(self) -> Result<super::GlobalStateStrParams<'static>> {
+        let hash_value =
+            unsafe_str_arg(self.hash_value, "casper_global_state_params_t.hash_value")?;
+        Ok(super::GlobalStateStrParams {
+            hash_value,
+            is_block_hash: self.is_block_hash,
+        })
+    }
+}
+
+/// Container for `DictionaryItemStrParams` construction options.
+///
+/// See [DictionaryItemStrParams](super::DictionaryItemStrParams) for more info.
+#[allow(missing_docs)]
+#[repr(C)]
+#[derive(Clone)]
+pub enum casper_dictionary_params_t {
+    AccountNamedKey {
+        key: *const c_char,
+        dictionary_name: *const c_char,
+        dictionary_item_key: *const c_char,
+    },
+    ContractNamedKey {
+        key: *const c_char,
+        dictionary_name: *const c_char,
+        dictionary_item_key: *const c_char,
+    },
+    URef {
+        seed_uref: *const c_char,
+        dictionary_item_key: *const c_char,
+    },
+    Dictionary(*const c_char),
+}
+
+impl TryInto<super::DictionaryItemStrParams<'static>> for casper_dictionary_params_t {
+    type Error = Error;
+
+    fn try_into(self) -> Result<super::DictionaryItemStrParams<'static>> {
+        match self {
+            casper_dictionary_params_t::AccountNamedKey {
+                key,
+                dictionary_name,
+                dictionary_item_key,
+            } => {
+                let key = unsafe_str_arg(key, "casper_dictionary_params_t::AccountNamedKey.key")?;
+                let dictionary_name = unsafe_str_arg(
+                    dictionary_name,
+                    "casper_dictionary_params_t::AccountNamedKey.dictionary_name",
+                )?;
+                let dictionary_item_key = unsafe_str_arg(
+                    dictionary_item_key,
+                    "casper_dictionary_params_t::AccountNamedKey.dictionary_item_key",
+                )?;
+                Ok(super::DictionaryItemStrParams::AccountNamedKey {
+                    key,
+                    dictionary_name,
+                    dictionary_item_key,
+                })
+            }
+            casper_dictionary_params_t::ContractNamedKey {
+                key,
+                dictionary_name,
+                dictionary_item_key,
+            } => {
+                let key = unsafe_str_arg(key, "casper_dictionary_params_t::ContractNamedKey.key")?;
+                let dictionary_name = unsafe_str_arg(
+                    dictionary_name,
+                    "casper_dictionary_params_t::ContractNamedKey.dictionary_name",
+                )?;
+                let dictionary_item_key = unsafe_str_arg(
+                    dictionary_item_key,
+                    "casper_dictionary_params_t::ContractNamedKey.dictionary_item_key",
+                )?;
+                Ok(super::DictionaryItemStrParams::ContractNamedKey {
+                    key,
+                    dictionary_name,
+                    dictionary_item_key,
+                })
+            }
+            casper_dictionary_params_t::URef {
+                seed_uref,
+                dictionary_item_key,
+            } => {
+                let seed_uref =
+                    unsafe_str_arg(seed_uref, "casper_dictionary_params_t::URef.seed_uref")?;
+                let dictionary_item_key = unsafe_str_arg(
+                    dictionary_item_key,
+                    "casper_dictionary_params_t::URef.dictionary_item_key",
+                )?;
+                Ok(super::DictionaryItemStrParams::URef {
+                    seed_uref,
+                    dictionary_item_key,
+                })
+            }
+            casper_dictionary_params_t::Dictionary(dictionary_address) => {
+                let dictionary_address =
+                    unsafe_str_arg(dictionary_address, "casper_dictionary_params_t::Dictionary")?;
+                Ok(super::DictionaryItemStrParams::Dictionary(
+                    dictionary_address,
+                ))
+            }
+        }
+    }
+}
