@@ -1,8 +1,8 @@
 //! # Casper node client library
 #![doc(
     html_root_url = "https://docs.rs/casper-client/0.1.0",
-    html_favicon_url = "https://raw.githubusercontent.com/casper-network/casper-node/master/images/CasperLabs_Logo_Favicon_RGB_50px.png",
-    html_logo_url = "https://raw.githubusercontent.com/casper-network/casper-node/master/images/CasperLabs_Logo_Symbol_RGB.png",
+    html_favicon_url = "https://raw.githubusercontent.com/CasperLabs/casper-node/master/images/CasperLabs_Logo_Favicon_RGB_50px.png",
+    html_logo_url = "https://raw.githubusercontent.com/CasperLabs/casper-node/master/images/CasperLabs_Logo_Symbol_RGB.png",
     test(attr(forbid(warnings)))
 )]
 #![warn(
@@ -24,19 +24,25 @@ mod validation;
 
 use std::{convert::TryInto, fs, io::Cursor};
 
+use hex::FromHex;
 use jsonrpc_lite::JsonRpc;
 use serde::Serialize;
 
 use casper_execution_engine::core::engine_state::ExecutableDeployItem;
-use casper_node::{rpcs::state::DictionaryIdentifier, types::Deploy};
-use casper_types::{Key, UIntParseError, U512};
+use casper_hashing::Digest;
+use casper_node::{
+    crypto,
+    rpcs::state::{DictionaryIdentifier, GlobalStateIdentifier},
+    types::{BlockHash, Deploy},
+};
+use casper_types::Key;
 
 pub use cl_type::help;
 pub use deploy::ListDeploysResult;
 use deploy::{DeployExt, DeployParams, OutputKind};
 pub use error::Error;
 use error::Result;
-use rpc::{RpcCall, TransferTarget};
+use rpc::RpcCall;
 pub use validation::ValidateResponseError;
 
 /// Creates a `Deploy` and sends it to the network for execution.
@@ -57,7 +63,7 @@ pub use validation::ValidateResponseError;
 ///   [`SessionStrParams`](struct.SessionStrParams.html) for more details.
 /// * `payment_params` contains payment-related options for this `Deploy`. See
 ///   [`PaymentStrParams`](struct.PaymentStrParams.html) for more details.
-pub fn put_deploy(
+pub async fn put_deploy(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
@@ -70,7 +76,9 @@ pub fn put_deploy(
         payment_params.try_into()?,
         session_params.try_into()?,
     )?;
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).put_deploy(deploy)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .put_deploy(deploy)
+        .await
 }
 
 /// Creates a `Deploy` and outputs it to a file or stdout.
@@ -160,13 +168,15 @@ pub fn sign_deploy_file(
 ///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
 ///   will not be printed to `stdout`.
 /// * `input_path` specifies the path to the previously-saved `Deploy` file.
-pub fn send_deploy_file(
+pub async fn send_deploy_file(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
     input_path: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).send_deploy_file(input_path)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .send_deploy_file(input_path)
+        .await
 }
 
 /// Transfers funds between purses.
@@ -182,8 +192,9 @@ pub fn send_deploy_file(
 ///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
 ///   will not be printed to `stdout`.
 /// * `amount` is a string to be parsed as a `U512` specifying the amount to be transferred.
-/// * `target_account` is the account `PublicKey` into which the funds will be transferred,
-///   formatted as a hex-encoded string. The account's main purse will receive the funds.
+/// * `target_account` is the `AccountHash`, `URef` or `PublicKey` of the account to which the funds
+///   will be transferred, formatted as a hex-encoded string. The account's main purse will receive
+///   the funds.
 /// * `transfer_id` is a string to be parsed as a `u64` representing a user-defined identifier which
 ///   will be permanently associated with the transfer.
 /// * `deploy_params` contains deploy-related options for this `Deploy`. See
@@ -191,7 +202,7 @@ pub fn send_deploy_file(
 /// * `payment_params` contains payment-related options for this `Deploy`. See
 ///   [`PaymentStrParams`](struct.PaymentStrParams.html) for more details.
 #[allow(clippy::too_many_arguments)]
-pub fn transfer(
+pub async fn transfer(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
@@ -201,20 +212,16 @@ pub fn transfer(
     deploy_params: DeployStrParams<'_>,
     payment_params: PaymentStrParams<'_>,
 ) -> Result<JsonRpc> {
-    let amount = U512::from_dec_str(amount)
-        .map_err(|err| Error::FailedToParseUint("amount", UIntParseError::FromDecStr(err)))?;
-    let source_purse = None;
-    let target = parsing::get_transfer_target(target_account)?;
-    let transfer_id = parsing::transfer_id(transfer_id)?;
-
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).transfer(
-        amount,
-        source_purse,
-        target,
-        transfer_id,
-        deploy_params.try_into()?,
-        payment_params.try_into()?,
-    )
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .transfer(
+            amount,
+            None,
+            target_account,
+            transfer_id,
+            deploy_params.try_into()?,
+            payment_params.try_into()?,
+        )
+        .await
 }
 
 /// Creates a transfer `Deploy` and outputs it to a file or stdout.
@@ -225,8 +232,9 @@ pub fn transfer(
 ///
 /// * `maybe_output_path` specifies the output file, or if empty, will print it to `stdout`.
 /// * `amount` is a string to be parsed as a `U512` specifying the amount to be transferred.
-/// * `target_account` is the account `PublicKey` into which the funds will be transferred,
-///   formatted as a hex-encoded string. The account's main purse will receive the funds.
+/// * `target_account` is the `AccountHash`, `URef` or `PublicKey` of the account to which the funds
+///   will be transferred, formatted as a hex-encoded string. The account's main purse will receive
+///   the funds.
 /// * `transfer_id` is a string to be parsed as a `u64` representing a user-defined identifier which
 ///   will be permanently associated with the transfer.
 /// * `deploy_params` contains deploy-related options for this `Deploy`. See
@@ -246,12 +254,6 @@ pub fn make_transfer(
     payment_params: PaymentStrParams<'_>,
     force: bool,
 ) -> Result<()> {
-    let amount = U512::from_dec_str(amount)
-        .map_err(|err| Error::FailedToParseUint("amount", UIntParseError::FromDecStr(err)))?;
-    let source_purse = None;
-    let target = parsing::get_transfer_target(target_account)?;
-    let transfer_id = parsing::transfer_id(transfer_id)?;
-
     let output = if maybe_output_path.is_empty() {
         OutputKind::Stdout
     } else {
@@ -260,8 +262,8 @@ pub fn make_transfer(
 
     Deploy::new_transfer(
         amount,
-        source_purse,
-        target,
+        None,
+        target_account,
         transfer_id,
         deploy_params.try_into()?,
         payment_params.try_into()?,
@@ -284,13 +286,15 @@ pub fn make_transfer(
 ///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
 ///   will not be printed to `stdout`.
 /// * `deploy_hash` must be a hex-encoded, 32-byte hash digest.
-pub fn get_deploy(
+pub async fn get_deploy(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
     deploy_hash: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_deploy(deploy_hash)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_deploy(deploy_hash)
+        .await
 }
 
 /// Retrieves a `Block` from the network.
@@ -307,13 +311,15 @@ pub fn get_deploy(
 ///   will not be printed to `stdout`.
 /// * `maybe_block_id` must be a hex-encoded, 32-byte hash digest or a `u64` representing the
 ///   `Block` height or empty. If empty, the latest `Block` will be retrieved.
-pub fn get_block(
+pub async fn get_block(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
     maybe_block_id: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_block(maybe_block_id)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_block(maybe_block_id)
+        .await
 }
 
 /// Retrieves all `Transfer` items for a `Block` from the network.
@@ -330,13 +336,15 @@ pub fn get_block(
 ///   will not be printed to `stdout`.
 /// * `maybe_block_id` must be a hex-encoded, 32-byte hash digest or a `u64` representing the
 ///   `Block` height or empty. If empty, the latest `Block` transfers will be retrieved.
-pub fn get_block_transfers(
+pub async fn get_block_transfers(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
     maybe_block_id: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_block_transfers(maybe_block_id)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_block_transfers(maybe_block_id)
+        .await
 }
 
 /// Retrieves a state root hash at a given `Block`.
@@ -353,13 +361,15 @@ pub fn get_block_transfers(
 ///   will not be printed to `stdout`.
 /// * `maybe_block_id` must be a hex-encoded, 32-byte hash digest or a `u64` representing the
 ///   `Block` height or empty. If empty, the latest `Block` will be used.
-pub fn get_state_root_hash(
+pub async fn get_state_root_hash(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
     maybe_block_id: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_state_root_hash(maybe_block_id)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_state_root_hash(maybe_block_id)
+        .await
 }
 
 /// Retrieves a stored value from the network.
@@ -379,15 +389,22 @@ pub fn get_state_root_hash(
 ///   or [`Key`](https://docs.rs/casper-types/latest/casper-types/enum.PublicKey.html). This will
 ///   take one of the following forms:
 /// ```text
-/// 01c9e33693951aaac23c49bee44ad6f863eedcd38c084a3a8f11237716a3df9c2c           # PublicKey
+/// 01c9e33693951aaac23c49bee44ad6f863eedcd38c084a3a8f11237716a3df9c2c             # PublicKey
 /// account-hash-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20  # Key::Account
-/// hash-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20        # Key::Hash
-/// uref-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20-007    # Key::URef
-/// transfer-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20    # Key::Transfer
-/// deploy-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20      # Key::DeployInfo
+/// hash-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20          # Key::Hash
+/// uref-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20-007      # Key::URef
+/// transfer-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20      # Key::Transfer
+/// deploy-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20        # Key::DeployInfo
+/// era-1                                                                          # Key::EraInfo
+/// bid-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20           # Key::Bid
+/// withdraw-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20      # Key::Withdraw
+/// dictionary-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20    # Key::Dictionary
+/// The Key::SystemContractRegistry variant is unique and can only take the following value:
+/// system-contract-registry-0000000000000000000000000000000000000000000000000000000000000000
 /// ```
 /// * `path` is comprised of components starting from the `key`, separated by `/`s.
-pub fn get_item(
+#[deprecated(note = "Users should use `casper_client::query_global_state` instead.")]
+pub async fn get_item(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
@@ -395,7 +412,9 @@ pub fn get_item(
     key: &str,
     path: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_item(state_root_hash, key, path)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_item(state_root_hash, key, path)
+        .await
 }
 
 /// Retrieves a purse's balance from the network.
@@ -415,14 +434,16 @@ pub fn get_item(
 /// ```text
 /// uref-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20-007
 /// ```
-pub fn get_balance(
+pub async fn get_balance(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
     state_root_hash: &str,
     purse: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_balance(state_root_hash, purse)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_balance(state_root_hash, purse)
+        .await
 }
 
 /// Retrieves era information from the network.
@@ -440,7 +461,7 @@ pub fn get_balance(
 /// * `maybe_block_id` must be a hex-encoded, 32-byte hash digest or a `u64` representing the
 ///   `Block` height or empty. If empty, era information from the latest block will be returned if
 ///   available.
-pub fn get_era_info_by_switch_block(
+pub async fn get_era_info_by_switch_block(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
@@ -448,6 +469,7 @@ pub fn get_era_info_by_switch_block(
 ) -> Result<JsonRpc> {
     RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
         .get_era_info_by_switch_block(maybe_block_id)
+        .await
 }
 
 /// Retrieves the bids and validators as of the most recently added `Block`.
@@ -465,13 +487,15 @@ pub fn get_era_info_by_switch_block(
 /// * `maybe_block_id` must be a hex-encoded, 32-byte hash digest or a `u64` representing the
 ///   `Block` height or empty. If empty, era information from the latest block will be returned if
 ///   available.
-pub fn get_auction_info(
+pub async fn get_auction_info(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
     maybe_block_id: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_auction_info(maybe_block_id)
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_auction_info(maybe_block_id)
+        .await
 }
 
 /// Retrieves an Account from the network.
@@ -489,7 +513,7 @@ pub fn get_auction_info(
 /// * `public_key` the public key associated with the `Account`
 /// * `maybe_block_id` must be a hex-encoded, 32-byte hash digest or a `u64` representing the
 ///   `Block` height or empty. If empty, the latest `Block` will be retrieved.
-pub fn get_account_info(
+pub async fn get_account_info(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
@@ -498,6 +522,52 @@ pub fn get_account_info(
 ) -> Result<JsonRpc> {
     RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
         .get_account_info(public_key, maybe_block_id)
+        .await
+}
+
+/// Retrieves information from global state using either a Block hash or a state root hash.
+///
+/// * `maybe_rpc_id` is the JSON-RPC identifier, applied to the request and returned in the
+///   response. If it can be parsed as an `i64` it will be used as a JSON integer. If empty, a
+///   random `i64` will be assigned. Otherwise the provided string will be used verbatim.
+/// * `node_address` is the hostname or IP and port of the node on which the HTTP service is
+///   running, e.g. `"http://127.0.0.1:7777"`.
+/// * When `verbosity_level` is `1`, the JSON-RPC request will be printed to `stdout` with long
+///   string fields (e.g. hex-formatted raw Wasm bytes) shortened to a string indicating the char
+///   count of the field.  When `verbosity_level` is greater than `1`, the request will be printed
+///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
+///   will not be printed to `stdout`.
+/// * `global_state_str_params` contains global state identifier related options for this query. See
+///   [`GlobalStateStrParams`](struct.GlobalStateStrParams.html) for more details.
+/// * `key` must be a formatted [`PublicKey`](https://docs.rs/casper-node/latest/casper-node/crypto/asymmetric_key/enum.PublicKey.html)
+///   or [`Key`](https://docs.rs/casper-types/latest/casper-types/enum.Key.html). This will take one
+///   of the following forms:
+/// ```text
+/// 01c9e33693951aaac23c49bee44ad6f863eedcd38c084a3a8f11237716a3df9c2c             # PublicKey
+/// account-hash-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20  # Key::Account
+/// hash-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20          # Key::Hash
+/// uref-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20-007      # Key::URef
+/// transfer-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20      # Key::Transfer
+/// deploy-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20        # Key::DeployInfo
+/// era-1                                                                          # Key::EraInfo
+/// bid-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20           # Key::Bid
+/// withdraw-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20      # Key::Withdraw
+/// dictionary-0102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f20    # Key::Dictionary
+/// The Key::SystemContractRegistry variant is unique and can only take the following value:
+/// system-contract-registry-0000000000000000000000000000000000000000000000000000000000000000
+/// ```
+/// * `path` is comprised of components starting from the `key`, separated by `/`s.
+pub async fn query_global_state(
+    maybe_rpc_id: &str,
+    node_address: &str,
+    verbosity_level: u64,
+    global_state_str_params: GlobalStateStrParams<'_>,
+    key: &str,
+    path: &str,
+) -> Result<JsonRpc> {
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .query_global_state(global_state_str_params, key, path)
+        .await
 }
 
 /// Retrieves information and examples for all currently supported RPCs.
@@ -512,11 +582,17 @@ pub fn get_account_info(
 ///   count of the field.  When `verbosity_level` is greater than `1`, the request will be printed
 ///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
 ///   will not be printed to `stdout`.
-pub fn list_rpcs(maybe_rpc_id: &str, node_address: &str, verbosity_level: u64) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).list_rpcs()
+pub async fn list_rpcs(
+    maybe_rpc_id: &str,
+    node_address: &str,
+    verbosity_level: u64,
+) -> Result<JsonRpc> {
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .list_rpcs()
+        .await
 }
 
-/// Retrieves a stored value from the network.
+/// Retrieves a stored value from a dictionary.
 ///
 /// * `maybe_rpc_id` is the JSON-RPC identifier, applied to the request and returned in the
 ///   response. If it can be parsed as an `i64` it will be used as a JSON integer. If empty, a
@@ -530,7 +606,7 @@ pub fn list_rpcs(maybe_rpc_id: &str, node_address: &str, verbosity_level: u64) -
 ///   will not be printed to `stdout`.
 /// * `state_root_hash` must be a hex-encoded, 32-byte hash digest.
 /// * `dictionary_str_params` contains options to query a dictionary item.
-pub fn get_dictionary(
+pub async fn get_dictionary_item(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
@@ -539,6 +615,29 @@ pub fn get_dictionary(
 ) -> Result<JsonRpc> {
     RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
         .get_dictionary_item(state_root_hash, dictionary_str_params)
+        .await
+}
+
+/// Retrieves status changes of active validators.
+///
+/// * `maybe_rpc_id` is the JSON-RPC identifier, applied to the request and returned in the
+///   response. If it can be parsed as an `i64` it will be used as a JSON integer. If empty, a
+///   random `i64` will be assigned. Otherwise the provided string will be used verbatim.
+/// * `node_address` is the hostname or IP and port of the node on which the HTTP service is
+///   running, e.g. `"http://127.0.0.1:7777"`.
+/// * When `verbosity_level` is `1`, the JSON-RPC request will be printed to `stdout` with long
+///   string fields (e.g. hex-formatted raw Wasm bytes) shortened to a string indicating the char
+///   count of the field.  When `verbosity_level` is greater than `1`, the request will be printed
+///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
+///   will not be printed to `stdout`.
+pub async fn get_validator_changes(
+    maybe_rpc_id: &str,
+    node_address: &str,
+    verbosity_level: u64,
+) -> Result<JsonRpc> {
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_validator_changes()
+        .await
 }
 
 /// Container for `Deploy` construction options.
@@ -571,6 +670,9 @@ pub struct DeployStrParams<'a> {
     /// Name of the chain, to avoid the `Deploy` from being accidentally or maliciously included in
     /// a different chain.
     pub chain_name: &'a str,
+    /// The hex-encoded public key of the account context under which the session code will be
+    /// executed.
+    pub session_account: &'a str,
 }
 
 impl<'a> TryInto<DeployParams> for DeployStrParams<'a> {
@@ -584,6 +686,7 @@ impl<'a> TryInto<DeployParams> for DeployStrParams<'a> {
             gas_price,
             dependencies,
             chain_name,
+            session_account,
         } = self;
         parsing::parse_deploy_params(
             secret_key,
@@ -592,6 +695,7 @@ impl<'a> TryInto<DeployParams> for DeployStrParams<'a> {
             gas_price,
             &dependencies,
             chain_name,
+            session_account,
         )
     }
 }
@@ -676,31 +780,7 @@ impl<'a> TryInto<ExecutableDeployItem> for PaymentStrParams<'a> {
     type Error = Error;
 
     fn try_into(self) -> Result<ExecutableDeployItem> {
-        let PaymentStrParams {
-            payment_amount,
-            payment_hash,
-            payment_name,
-            payment_package_hash,
-            payment_package_name,
-            payment_path,
-            payment_args_simple,
-            payment_args_complex,
-            payment_version,
-            payment_entry_point,
-        } = self;
-
-        parsing::parse_payment_info(
-            payment_amount,
-            payment_hash,
-            payment_name,
-            payment_package_hash,
-            payment_package_name,
-            payment_path,
-            &payment_args_simple,
-            payment_args_complex,
-            payment_version,
-            payment_entry_point,
-        )
+        parsing::parse_payment_info(self)
     }
 }
 
@@ -750,9 +830,9 @@ impl<'a> PaymentStrParams<'a> {
     ) -> Self {
         Self {
             payment_name,
-            payment_entry_point,
             payment_args_simple,
             payment_args_complex,
+            payment_entry_point,
             ..Default::default()
         }
     }
@@ -772,9 +852,9 @@ impl<'a> PaymentStrParams<'a> {
     ) -> Self {
         Self {
             payment_hash,
-            payment_entry_point,
             payment_args_simple,
             payment_args_complex,
+            payment_entry_point,
             ..Default::default()
         }
     }
@@ -797,10 +877,10 @@ impl<'a> PaymentStrParams<'a> {
     ) -> Self {
         Self {
             payment_package_name,
-            payment_version,
-            payment_entry_point,
             payment_args_simple,
             payment_args_complex,
+            payment_version,
+            payment_entry_point,
             ..Default::default()
         }
     }
@@ -824,10 +904,10 @@ impl<'a> PaymentStrParams<'a> {
     ) -> Self {
         Self {
             payment_package_hash,
-            payment_version,
-            payment_entry_point,
             payment_args_simple,
             payment_args_complex,
+            payment_version,
+            payment_entry_point,
             ..Default::default()
         }
     }
@@ -837,31 +917,7 @@ impl<'a> TryInto<ExecutableDeployItem> for SessionStrParams<'a> {
     type Error = Error;
 
     fn try_into(self) -> Result<ExecutableDeployItem> {
-        let SessionStrParams {
-            session_hash,
-            session_name,
-            session_package_hash,
-            session_package_name,
-            session_path,
-            session_args_simple,
-            session_args_complex,
-            session_version,
-            session_entry_point,
-            is_session_transfer,
-        } = self;
-
-        parsing::parse_session_info(
-            session_hash,
-            session_name,
-            session_package_hash,
-            session_package_name,
-            session_path,
-            &session_args_simple,
-            session_args_complex,
-            session_version,
-            session_entry_point,
-            is_session_transfer,
-        )
+        parsing::parse_session_info(self)
     }
 }
 
@@ -934,9 +990,9 @@ impl<'a> SessionStrParams<'a> {
     ) -> Self {
         Self {
             session_name,
-            session_entry_point,
             session_args_simple,
             session_args_complex,
+            session_entry_point,
             ..Default::default()
         }
     }
@@ -956,9 +1012,9 @@ impl<'a> SessionStrParams<'a> {
     ) -> Self {
         Self {
             session_hash,
-            session_entry_point,
             session_args_simple,
             session_args_complex,
+            session_entry_point,
             ..Default::default()
         }
     }
@@ -981,10 +1037,10 @@ impl<'a> SessionStrParams<'a> {
     ) -> Self {
         Self {
             session_package_name,
-            session_version,
-            session_entry_point,
             session_args_simple,
             session_args_complex,
+            session_version,
+            session_entry_point,
             ..Default::default()
         }
     }
@@ -1008,10 +1064,10 @@ impl<'a> SessionStrParams<'a> {
     ) -> Self {
         Self {
             session_package_hash,
-            session_version,
-            session_entry_point,
             session_args_simple,
             session_args_complex,
+            session_version,
+            session_entry_point,
             ..Default::default()
         }
     }
@@ -1114,6 +1170,33 @@ impl<'a> TryInto<DictionaryIdentifier> for DictionaryItemStrParams<'a> {
     }
 }
 
+/// The two ways to construct a query to global state.
+#[derive(Default, Debug)]
+pub struct GlobalStateStrParams<'a> {
+    /// Identifier to mark the hash as either a Block hash or `state_root_hash`
+    /// When true, the hash provided is a Block hash.
+    pub is_block_hash: bool,
+    /// The hex-encoded hash value.
+    pub hash_value: &'a str,
+}
+
+impl<'a> TryInto<GlobalStateIdentifier> for GlobalStateStrParams<'a> {
+    type Error = Error;
+
+    fn try_into(self) -> Result<GlobalStateIdentifier> {
+        let hash = Digest::from_hex(self.hash_value).map_err(|error| Error::CryptoError {
+            context: "global_state_identifier",
+            error: crypto::Error::FromHex(error),
+        })?;
+
+        if self.is_block_hash {
+            Ok(GlobalStateIdentifier::BlockHash(BlockHash::new(hash)))
+        } else {
+            Ok(GlobalStateIdentifier::StateRootHash(hash))
+        }
+    }
+}
+
 /// When `verbosity_level` is `1`, the value will be printed to `stdout` with long string fields
 /// (e.g. hex-formatted raw Wasm bytes) shortened to a string indicating the char count of the
 /// field.  When `verbosity_level` is greater than `1`, the value will be printed to `stdout` with
@@ -1141,21 +1224,6 @@ pub fn pretty_print_at_level<T: ?Sized + Serialize>(value: &T, verbosity_level:
 mod param_tests {
     use super::*;
 
-    #[derive(Debug)]
-    struct ErrWrapper(pub Error);
-
-    impl PartialEq for ErrWrapper {
-        fn eq(&self, other: &ErrWrapper) -> bool {
-            format!("{:?}", self.0) == format!("{:?}", other.0)
-        }
-    }
-
-    impl From<Error> for ErrWrapper {
-        fn from(error: Error) -> Self {
-            ErrWrapper(error)
-        }
-    }
-
     const HASH: &str = "09dcee4b212cfd53642ab323fbef07dafafc6f945a80a00147f62910a915c4e6";
     const NAME: &str = "name";
     const PKG_NAME: &str = "pkg_name";
@@ -1256,7 +1324,7 @@ mod param_tests {
     mod payment_params {
         use std::collections::BTreeMap;
 
-        use casper_types::CLValue;
+        use casper_types::{CLValue, U512};
 
         use super::*;
 
@@ -1371,8 +1439,7 @@ mod param_tests {
 
         #[test]
         fn should_convert_into_deploy_params() {
-            let deploy_params: StdResult<DeployParams, ErrWrapper> =
-                test_value().try_into().map_err(ErrWrapper);
+            let deploy_params: StdResult<DeployParams, _> = test_value().try_into();
             assert!(deploy_params.is_ok());
         }
 
@@ -1381,14 +1448,13 @@ mod param_tests {
             let mut params = test_value();
             params.timestamp = "garbage";
             let result: StdResult<DeployParams, Error> = params.try_into();
-            let result = result.map(|_| ()).map_err(ErrWrapper);
-            assert_eq!(
+            assert!(matches!(
                 result,
-                Err(
-                    Error::FailedToParseTimestamp("timestamp", TimestampError::InvalidFormat)
-                        .into()
-                )
-            );
+                Err(Error::FailedToParseTimestamp {
+                    context: "timestamp",
+                    error: TimestampError::InvalidFormat
+                })
+            ));
         }
 
         #[test]
@@ -1397,7 +1463,7 @@ mod param_tests {
             params.gas_price = "fifteen";
             let result: StdResult<DeployParams, Error> = params.try_into();
             let result = result.map(|_| ());
-            if let Err(Error::FailedToParseInt(context, _)) = result {
+            if let Err(Error::FailedToParseInt { context, error: _ }) = result {
                 assert_eq!(context, "gas_price");
             } else {
                 panic!("should be an error");
@@ -1409,8 +1475,7 @@ mod param_tests {
             let mut params = test_value();
             params.chain_name = "";
             let result: StdResult<DeployParams, Error> = params.try_into();
-            let result = result.map(|_| ()).map_err(ErrWrapper);
-            assert_eq!(result, Ok(()));
+            assert!(matches!(result, Ok(_)));
         }
 
         #[test]
@@ -1418,11 +1483,13 @@ mod param_tests {
             let mut params = test_value();
             params.ttl = "not_a_ttl";
             let result: StdResult<DeployParams, Error> = params.try_into();
-            let result = result.map(|_| ()).map_err(ErrWrapper);
-            assert_eq!(
+            assert!(matches!(
                 result,
-                Err(Error::FailedToParseTimeDiff("ttl", DurationError::NumberExpected(0)).into())
-            );
+                Err(Error::FailedToParseTimeDiff {
+                    context: "ttl",
+                    error: DurationError::NumberExpected(0)
+                })
+            ));
         }
 
         #[test]
@@ -1430,7 +1497,6 @@ mod param_tests {
             let mut params = test_value();
             params.secret_key = "";
             let result: StdResult<DeployParams, Error> = params.try_into();
-            let result = result.map(|_| ());
             if let Err(Error::CryptoError { context, .. }) = result {
                 assert_eq!(context, "secret_key");
             } else {
@@ -1444,15 +1510,13 @@ mod param_tests {
             let mut params = test_value();
             params.dependencies = vec!["invalid dep"];
             let result: StdResult<DeployParams, Error> = params.try_into();
-            let result = result.map(|_| ()).map_err(ErrWrapper);
-            assert_eq!(
+            assert!(matches!(
                 result,
                 Err(Error::CryptoError {
                     context: "dependencies",
                     error: CryptoError::FromHex(hex::FromHexError::OddLength)
-                }
-                .into())
-            );
+                })
+            ));
         }
     }
 }
