@@ -1,5 +1,6 @@
 use clap::{App, ArgMatches, SubCommand};
 
+use async_trait::async_trait;
 use casper_client::Error;
 
 use super::creation_common::{self, DisplayOrder};
@@ -7,6 +8,7 @@ use crate::{command::ClientCommand, common, Success};
 
 pub struct SendDeploy;
 
+#[async_trait]
 impl<'a, 'b> ClientCommand<'a, 'b> for SendDeploy {
     const NAME: &'static str = "send-deploy";
     const ABOUT: &'static str =
@@ -24,13 +26,14 @@ impl<'a, 'b> ClientCommand<'a, 'b> for SendDeploy {
             .arg(creation_common::input::arg())
     }
 
-    fn run(matches: &ArgMatches<'_>) -> Result<Success, Error> {
+    async fn run(matches: &ArgMatches<'a>) -> Result<Success, Error> {
         let maybe_rpc_id = common::rpc_id::get(matches);
         let node_address = common::node_address::get(matches);
         let verbosity_level = common::verbose::get(matches);
         let input_path = creation_common::input::get(matches);
 
         casper_client::send_deploy_file(maybe_rpc_id, node_address, verbosity_level, input_path)
+            .await
             .map(Success::from)
     }
 }
