@@ -8,11 +8,12 @@ mod generate_completion;
 mod get_account_info;
 mod get_auction_info;
 mod get_balance;
+mod get_dictionary_item;
 mod get_era_info_by_switch_block;
 mod get_state_hash;
+mod get_validator_changes;
 mod keygen;
-mod query_dictionary;
-mod query_state;
+mod query_global_state;
 
 use std::process;
 
@@ -23,8 +24,8 @@ use casper_node::rpcs::{
     account::PutDeploy,
     chain::{GetBlock, GetBlockTransfers, GetEraInfoBySwitchBlock, GetStateRootHash},
     docs::ListRpcs,
-    info::GetDeploy,
-    state::{GetAccountInfo, GetAuctionInfo, GetBalance, GetDictionaryItem, GetItem as QueryState},
+    info::{GetDeploy, GetValidatorChanges},
+    state::{GetAccountInfo, GetAuctionInfo, GetBalance, GetDictionaryItem, QueryGlobalState},
 };
 
 use account_address::GenerateAccountHash as AccountAddress;
@@ -48,16 +49,17 @@ enum DisplayOrder {
     GetBlockTransfers,
     ListDeploys,
     GetStateRootHash,
-    QueryState,
+    QueryGlobalState,
+    GetDictionaryItem,
     GetBalance,
     GetAccountInfo,
     GetEraInfo,
     GetAuctionInfo,
+    GetValidatorChanges,
     Keygen,
     GenerateCompletion,
     GetRpcs,
     AccountAddress,
-    GetDictionaryItem,
 }
 
 fn cli<'a, 'b>() -> App<'a, 'b> {
@@ -81,11 +83,13 @@ fn cli<'a, 'b>() -> App<'a, 'b> {
         .subcommand(GetStateRootHash::build(
             DisplayOrder::GetStateRootHash as usize,
         ))
-        .subcommand(QueryState::build(DisplayOrder::QueryState as usize))
         .subcommand(GetEraInfoBySwitchBlock::build(
             DisplayOrder::GetEraInfo as usize,
         ))
         .subcommand(GetAuctionInfo::build(DisplayOrder::GetAuctionInfo as usize))
+        .subcommand(GetValidatorChanges::build(
+            DisplayOrder::GetValidatorChanges as usize,
+        ))
         .subcommand(Keygen::build(DisplayOrder::Keygen as usize))
         .subcommand(GenerateCompletion::build(
             DisplayOrder::GenerateCompletion as usize,
@@ -95,35 +99,48 @@ fn cli<'a, 'b>() -> App<'a, 'b> {
         .subcommand(GetDictionaryItem::build(
             DisplayOrder::GetDictionaryItem as usize,
         ))
+        .subcommand(
+            QueryGlobalState::build(DisplayOrder::QueryGlobalState as usize).alias("query-state"),
+        )
 }
 
 #[tokio::main]
 async fn main() {
     let arg_matches = cli().get_matches();
     let (result, matches) = match arg_matches.subcommand() {
-        (PutDeploy::NAME, Some(matches)) => (PutDeploy::run(matches), matches),
-        (MakeDeploy::NAME, Some(matches)) => (MakeDeploy::run(matches), matches),
-        (SignDeploy::NAME, Some(matches)) => (SignDeploy::run(matches), matches),
-        (SendDeploy::NAME, Some(matches)) => (SendDeploy::run(matches), matches),
-        (Transfer::NAME, Some(matches)) => (Transfer::run(matches), matches),
-        (MakeTransfer::NAME, Some(matches)) => (MakeTransfer::run(matches), matches),
-        (GetDeploy::NAME, Some(matches)) => (GetDeploy::run(matches), matches),
-        (GetBlock::NAME, Some(matches)) => (GetBlock::run(matches), matches),
-        (GetBlockTransfers::NAME, Some(matches)) => (GetBlockTransfers::run(matches), matches),
-        (ListDeploys::NAME, Some(matches)) => (ListDeploys::run(matches), matches),
-        (GetBalance::NAME, Some(matches)) => (GetBalance::run(matches), matches),
-        (GetAccountInfo::NAME, Some(matches)) => (GetAccountInfo::run(matches), matches),
-        (GetStateRootHash::NAME, Some(matches)) => (GetStateRootHash::run(matches), matches),
-        (QueryState::NAME, Some(matches)) => (QueryState::run(matches), matches),
-        (GetEraInfoBySwitchBlock::NAME, Some(matches)) => {
-            (GetEraInfoBySwitchBlock::run(matches), matches)
+        (PutDeploy::NAME, Some(matches)) => (PutDeploy::run(matches).await, matches),
+        (MakeDeploy::NAME, Some(matches)) => (MakeDeploy::run(matches).await, matches),
+        (SignDeploy::NAME, Some(matches)) => (SignDeploy::run(matches).await, matches),
+        (SendDeploy::NAME, Some(matches)) => (SendDeploy::run(matches).await, matches),
+        (Transfer::NAME, Some(matches)) => (Transfer::run(matches).await, matches),
+        (MakeTransfer::NAME, Some(matches)) => (MakeTransfer::run(matches).await, matches),
+        (GetDeploy::NAME, Some(matches)) => (GetDeploy::run(matches).await, matches),
+        (GetBlock::NAME, Some(matches)) => (GetBlock::run(matches).await, matches),
+        (GetBlockTransfers::NAME, Some(matches)) => {
+            (GetBlockTransfers::run(matches).await, matches)
         }
-        (GetAuctionInfo::NAME, Some(matches)) => (GetAuctionInfo::run(matches), matches),
-        (Keygen::NAME, Some(matches)) => (Keygen::run(matches), matches),
-        (GenerateCompletion::NAME, Some(matches)) => (GenerateCompletion::run(matches), matches),
-        (ListRpcs::NAME, Some(matches)) => (ListRpcs::run(matches), matches),
-        (AccountAddress::NAME, Some(matches)) => (AccountAddress::run(matches), matches),
-        (GetDictionaryItem::NAME, Some(matches)) => (GetDictionaryItem::run(matches), matches),
+        (ListDeploys::NAME, Some(matches)) => (ListDeploys::run(matches).await, matches),
+        (GetBalance::NAME, Some(matches)) => (GetBalance::run(matches).await, matches),
+        (GetAccountInfo::NAME, Some(matches)) => (GetAccountInfo::run(matches).await, matches),
+        (GetStateRootHash::NAME, Some(matches)) => (GetStateRootHash::run(matches).await, matches),
+        (GetEraInfoBySwitchBlock::NAME, Some(matches)) => {
+            (GetEraInfoBySwitchBlock::run(matches).await, matches)
+        }
+        (GetAuctionInfo::NAME, Some(matches)) => (GetAuctionInfo::run(matches).await, matches),
+        (GetValidatorChanges::NAME, Some(matches)) => {
+            (GetValidatorChanges::run(matches).await, matches)
+        }
+        (Keygen::NAME, Some(matches)) => (Keygen::run(matches).await, matches),
+        (GenerateCompletion::NAME, Some(matches)) => {
+            (GenerateCompletion::run(matches).await, matches)
+        }
+        (ListRpcs::NAME, Some(matches)) => (ListRpcs::run(matches).await, matches),
+        (AccountAddress::NAME, Some(matches)) => (AccountAddress::run(matches).await, matches),
+        (GetDictionaryItem::NAME, Some(matches)) => {
+            (GetDictionaryItem::run(matches).await, matches)
+        }
+        (QueryGlobalState::NAME, Some(matches)) => (QueryGlobalState::run(matches).await, matches),
+
         _ => {
             let _ = cli().print_long_help();
             println!();
